//
//  HerasController.m
//  musicplayer
//
//  Created by jianquan chen on 12-6-17.
//  Copyright (c) 2012年 jianquan chen. All rights reserved.
//

#import "HerasController.h"
#import "HerasTime.h"
@implementation HerasController
@synthesize randomButton;
@synthesize  progressbar;
@synthesize  volumbar;
@synthesize  name;
@synthesize  time;
@synthesize  table;
@synthesize  playPause;
@synthesize  volumeButton;
@synthesize  loopButton;
@synthesize indicator;
-(id) init
{
    if(self=[super init])
    {
        [self myInit];   
    }
    return self;
}
-(void) myInit
{
    fileTypes=[QTMovie movieFileTypes:0];
    player=[[HerasPlayer alloc] init];
    player.delegate=self;
    ds=[[HerasArrayDataSource alloc]init];
    ready=YES;
    timer=[NSTimer scheduledTimerWithTimeInterval:1 target:self selector:@selector(updatePlayProgress:) userInfo:nil repeats:YES];
    [timer setFireDate:[NSDate distantFuture]];
}

-(void) pauseTimer
{
    if([timer isValid])
    {
        [timer setFireDate:[NSDate distantFuture]];
    }
}

-(void) resumeTimer
{
    if([timer isValid])
    {
        [timer setFireDate:[NSDate date]];
    }
}
-(void) updatePlayProgress:(NSTimer *)timer
{
    [progressbar setFloatValue:[player position]];
    [time setStringValue:[HerasTime parse:[player currentTime]]];
    
}
-(IBAction)volumeUpDown:(NSMenuItem *)sender
{
    if([player isPlaying]){
        NSString * title=[sender title];
        float volume=[volumbar floatValue];
        if ([@"Increase" isEqualToString:title]) {
            volume+=0.1;
            if(volume>1)
            {
                volume=1;
            }
        }
        else {
            volume-=0.1;
            if(volume<0)
            {
                volume=0;
            }
        }
        [volumbar setFloatValue:volume];
        [self volumeChange:nil];
    }
}
-(IBAction)modeChange:(NSMenuItem *)sender
{
    NSString * title=[sender title];
    if ([@"Single" isEqualToString:title]) {
        [randomButton setState:NSOffState];
        [loopButton setState: NSOffState];
    }
    else if([@"Random" isEqualToString:title]) {
        [randomButton setState:NSOnState];
        [loopButton setState:NSOffState];
    }
    else {
        [randomButton setState:NSOffState];
        [loopButton setState:NSOnState];
    }
    
}
-(IBAction)fopen:(id)sender
{
    [self initlizeFileManager];
    if([openPanel runModal]==NSOKButton)
    {
        NSThread* thread=[[NSThread alloc] initWithTarget:self selector:@selector(addSounds:) object:[openPanel URLs]];
        [thread start];
        
    }
}
-(void) addSounds:(NSArray *) arr
{
    total=[arr count];
    for(NSURL* url in arr)
    {
        [player add: url];
        [self performSelectorOnMainThread:@selector(updateList) withObject:nil waitUntilDone:NO];        
    }
    [player save];
    total=0;
    [self notify];
}
-(void) notify
{
    if (total>0) {
        [indicator setHidden:NO];
        double percentage=100.0/total;
        [indicator incrementBy:percentage];
    }
    else {
        [indicator setHidden:YES];
        [indicator setDoubleValue:0];
    }
}
-(void) updateList
{
    if([table dataSource]){
        [table reloadData];
    }
    else {
        [table setDataSource:ds];
    }
    [self notify];
    
}
-(void) awakeFromNib
{
    NSBundle *bundle=[NSBundle mainBundle];
    //button play  image
    imagePlay=[[NSImage alloc] initWithContentsOfFile:[bundle pathForResource:@"player_play" ofType:@"png"]];
    //button pause image
    imagePause=[[NSImage alloc] initWithContentsOfFile:[bundle pathForResource:@"player_pause" ofType:@"png"]];
    //button volume  image
    imageVolume=[[NSImage alloc] initWithContentsOfFile:[bundle pathForResource:@"player_volume" ofType:@"png"]];
    //button volume mute image
    imageVolumeMute=[[NSImage alloc] initWithContentsOfFile:[bundle pathForResource:@"player_volume_mute" ofType:@"png"]];
   
    [table setTarget:self];
    [table setDoubleAction:@selector(jumpToPlay:)];
    [table setAllowsEmptySelection:NO];
    [player load];
    [ds setDataSource:[player playlist]];
    [table setDataSource:ds];
    [indicator setHidden:YES];
    [indicator setDoubleValue:0];
}
-(void) initlizeFileManager
{
    if(openPanel==nil)
    {
        openPanel=[NSOpenPanel openPanel];
        [openPanel setCanCreateDirectories:NO];
        [openPanel setAllowsMultipleSelection:YES];
        [openPanel setAllowedFileTypes:fileTypes];
        [openPanel setCanChooseFiles:YES];
        [openPanel setCanChooseDirectories:NO];
        [openPanel setCanSelectHiddenExtension:NO];
    }
}
-(IBAction)mute:(id *)sender
{
    if([player isPlaying]){
        if([player muted])
        {
            [player unmute];
            [volumbar setFloatValue:[player volume]];
            [volumeButton setImage:imageVolume];
        }
        else {
            [player mute];
            [volumbar setFloatValue:0];
            [volumeButton setImage:imageVolumeMute];
        }
    }
}

-(IBAction)playClick:(id *)sender
{
    
    [self pauseTimer];
    if (ready) {
        [player setVolume:[volumbar floatValue]];
        [player play];
        if([player isPlaying])
        {
            ready=NO;
            [playPause setImage:imagePause];
            [name setStringValue:[player currentSound] ];
            [self resumeTimer];
        }
    }
    else {
        ready=YES;
        [player stop];
        [playPause setImage:imagePlay];
        
    }
    
}
-(IBAction)nextClick:(id *)sender
{
    [self pauseTimer];
    [progressbar setIntValue:0];
    [player next];
    if([player isPlaying]){
        ready=NO;
        [playPause setImage:imagePause];
        [name setStringValue:[player currentSound] ];
        [self resumeTimer];
    }
}
-(IBAction)previousClick:(id *)sender
{
    
    [self pauseTimer];
    [progressbar setIntValue:0];
    [player previous];
    if([player isPlaying]){
        ready=NO;
        [playPause setImage:imagePause];
        [name setStringValue:[player currentSound] ];
        [self resumeTimer];
    }
    
}
-(IBAction)checkboxCheck:(id)sender{
    NSString * identifier=[sender identifier];
    if([@"random" isEqualToString:identifier])
    {
        if([randomButton state]==NSOnState)
        {
            [loopButton setState:NSOffState];
        }
    }
    else {
        if([loopButton state]==NSOnState )
        {
            [randomButton setState:NSOffState];
        }
    }
    
    
}
-(void) playbackComplete:(id) param
{
    
    [self pauseTimer];
    [progressbar setIntValue:0];
    if(([loopButton state]==NSOnState)||([randomButton state]==NSOnState))
    {
        if([loopButton state]==NSOnState)
        {
            [player next];
        }
        else
        {
            [player randmom];
        }
        if ([player isPlaying]) {
            [name setStringValue:[player currentSound] ];
            [self resumeTimer];
        }
        
    }
    else {
        if (NO==[player isPlaying]) {
            ready=YES;
            [playPause setImage:imagePlay];
            [name setStringValue:@""];
            [time setStringValue:@"00:00:00"];
        }
    }
    
}
-(void)jumpToPlay:(NSTableView *)sender{
    
    [self pauseTimer];
    [progressbar setFloatValue:0];
    [player setVolume:[volumbar floatValue]];
    [player jumpTo:[table selectedRow]];
    NSString *sname=[player currentSound];
    if(sname){
        [name setStringValue:sname];
    }
    if([player isPlaying])
    {
        [playPause setImage:imagePause];
        ready=NO;
        [self resumeTimer];
    }
}

-(IBAction)volumeChange:(id *)sender
{
    if ([player isPlaying]) 
    {
        [player setVolume:[volumbar floatValue]];
        if([player muted]||[player volume]<=0)
        {
            [volumeButton setImage:imageVolumeMute];
        }
        else {
            [volumeButton setImage:imageVolume];
        }
    }
}
-(IBAction)deleteAll:(id)sender
{
    if ([player size]>0) {
        [self pauseTimer];
        [progressbar setFloatValue:0];
        [time setStringValue:@"00:00:00"];
        [name setStringValue:@""];
        [player empty];
        [player save];
        [table reloadData];
        [playPause setImage:imagePlay];
        ready=YES;
    }
}

-(IBAction)deleteOne:(id)sender
{
    if ([player size]>0) {
        NSInteger row=[table selectedRow];
        if (row>=0) {
            [player removeAt:row];
            [table reloadData];
            [player save];
            if (NO==[player isPlaying]) {
                [self pauseTimer];
                [progressbar setFloatValue:0];
                [time setStringValue:@"00:00:00"];
                [name setStringValue:@""];
                [playPause setImage:imagePlay];
                ready=YES;
            }
            
        }
        
        
        
    }
    
}
-(IBAction)playbackChange:(id *)sender
{
    if([player isPlaying]){
        [self pauseTimer];
        [player seek:[progressbar floatValue]];
        if ([player isPlaying]) {
            [self resumeTimer];  
        }
    }
}
-(IBAction)mAbout:(id)sender
{
    if(alert==nil){
        alert=[[NSAlert alloc] init];
        [alert addButtonWithTitle:@"OK"];
        [alert setInformativeText:@"MusicPlayer is a simple music player developed by heras(quan861122@gmail.com)"];
        [alert setMessageText:@"About MusicPlayer"];
        [alert setAlertStyle:NSInformationalAlertStyle];
        
    }
    [alert runModal];
}
@end
