//
//  HerasSoundPlayer.m
//  musicplayer
//
//  Created by heras on 12-6-18.
//  Copyright (c) 2012年 heras(quan861122@gmail.com). All rights reserved.
//

#import "HerasPlayer.h"

@implementation HerasPlayer
@synthesize delegate;
-(id) init
{
    if(self=[super init])
    {
        list=nil;
        index=0;
        _volume=0.5;
    }
    return self;
}
-(NSString *) currentSound
{
    NSString* csound=@"";
    if ([self current]!=nil) {
        NSURL * sound=[[self current] url] ;
        NSRange range=[[sound relativePath] rangeOfString:@"/" options:NSBackwardsSearch];
        NSString * filename=[[sound relativePath] substringFromIndex:range.location+1];
        range=[filename rangeOfString:@"."];
        csound=[filename substringToIndex:range.location];
    }
    return csound;
}
-(float) position
{
    float pos=0;
    if (movie) {
        QTTime duration=[movie duration];
        QTTime current=[movie currentTime];
        pos=(float)current.timeValue/duration.timeValue;
    }
    return pos;
}
-(void) playComplete:(NSNotification *) notification
{
    playing=NO;
    [delegate playbackComplete:[notification object]]; 
}

-(void) load
{
    @synchronized(self)
    {
        NSArray * array= [[NSArray alloc] initWithContentsOfFile:[@"~/.musicplayer/playlist.xml" stringByExpandingTildeInPath]];
        if (array!=nil) 
        {
            if (list==nil) 
            {
                list=[[NSMutableArray alloc]  initWithCapacity:[array count]];
            }
            for (NSString * path in array)
            {
                HerasSound *sound=[[HerasSound alloc] initWithURL:[NSURL fileURLWithPath:path]];
                if(sound)
                {
                    [list addObject:sound];
                    sound.index=[list count];
                }
            }
        }
    }
    
}
-(BOOL) save
{
    @synchronized(self){
        BOOL saved=NO;
        NSMutableArray * files=[NSMutableArray arrayWithCapacity:[list count]];
        for(HerasSound * sound in list)
        {
            [files addObject:[[sound url] relativePath] ];
        }
        NSString *path=[@"~/.musicplayer" stringByExpandingTildeInPath];
        NSFileManager * fmanager=[NSFileManager defaultManager];
        
        BOOL directory=YES;
        if([fmanager fileExistsAtPath:path isDirectory:&directory])
        {
            saved=[files writeToFile:[@"~/.musicplayer/playlist.xml" stringByExpandingTildeInPath] atomically:YES];
        }
        else {
            BOOL success=[fmanager createDirectoryAtPath:path withIntermediateDirectories:YES attributes:nil error:nil];
            if(success){
                saved=[files writeToFile:[@"~/.musicplayer/playlist.xml" stringByExpandingTildeInPath] atomically:YES];
            }
            else {
                return  NO;
            }
        }
        return saved;
    }
}
-(NSMutableArray *) playlist
{
    return list;
}
-(NSInteger) currentIndex
{
    return index;
}
-(HerasSound*) current
{
    return _current;
}
-(void) add:(NSURL *) file
{
    @synchronized(self)
    {
        
        if(list==nil)
        {
            list=[[NSMutableArray alloc] initWithCapacity:5];
        }
        if(file)
        {
            HerasSound *sound=[[HerasSound alloc] initWithURL:file];
            if(sound)
            {
                [list addObject:sound];
                [sound setIndex:[list count]];
            }
        }
    }
}
-(void)  empty
{
    @synchronized(self)
    {
        if ([list count]>0) {
            [self stop];
            index=0;
            [list removeAllObjects];
        }
    }
}
-(void) removeAt:(NSInteger)pos
{
    @synchronized(self){
        if ([list count]>0) 
        {
            if(pos>=0&&pos<[list count])
            {
                if (pos==index) {
                    [self stop];
                    index=0;
                }
                [list removeObjectAtIndex:pos];
                [self reIndex];
            }
        }
    }
}
-(void) reIndex
{
    for (int i=0; i<[list count]; i++) {
        HerasSound * sound=[list objectAtIndex:i];
        sound.index=i+1;
        if ([_current equals:sound]) {
            index=sound.index-1;
        }
    }
    
}
-(NSInteger) size
{
    return [list count]; 
}

-(void) setVolume:(float)volume
{
    if (volume<0) {
        _volume=0;
    }
    if(volume>1)
    {
        _volume=1;
    }
    _volume=volume;
    [movie setVolume:volume];
    
    
}

-(float) volume
{
    return [movie volume];
}
-(void) next
{
    [self stop];
    index++;
    if (index>[list count]-1) {
        index=0;
    }
    [self playAt:index];  
    
}
-(BOOL) isPlaying
{
    return playing;
}
-(void) previous
{
    [self stop];
    index--;
    if (index<0) {
        index=[list count]-1;
    }
    [self playAt:index];
    
}

-(void) play
{
    [self playAt:index];
}
-(void) playAt:(NSInteger) pos
{
    if(pos>=0&&pos<[list count])
    {
        if(NO==playing)
        {
            index=pos;
            [movie stop];
            if(center==nil) {
                center=[NSNotificationCenter defaultCenter];
            }
            [center removeObserver:self];
            movie=nil;
            _current=[list objectAtIndex:index];
            movie=[_current movie];
            [center addObserver:self selector:@selector(playComplete:) name:QTMovieDidEndNotification object:movie];
            [movie setVolume:_volume];
            [movie play];
            if(movie)
            {
                playing=YES;
            }
        }
    }
}

-(void) stop
{
    [movie stop];
    playing=NO;
}
-(void) mute
{
    [movie setMuted:YES];
    _volume=0;
}
-(void) unmute
{
    [movie setMuted:NO];
    _volume=[movie volume];
}
-(void) randmom
{
    if ([list count]>0) {
        [self playAt:arc4random()%[list count]];
    }
}
-(void) seek:(float)position
{
    if(position>=0&&position<=1)
    {
        QTTime time=QTMakeTime((long long)(position*[movie duration].timeValue), [movie duration].timeScale);
        [movie setCurrentTime:time];
        [movie setVolume:_volume];
    }
}

-(void) jumpTo:(NSInteger)newIndex
{
    [self stop];
    [self playAt:newIndex];
}
-(QTTime) currentTime
{
    return [movie currentTime];
}
-(QTTime) duration
{
    return [movie duration];
}
-(BOOL) muted
{
    return [movie muted];
}

@end
